/* ==========================================================================
 * $File: //dwh/usb_iip/dev/software/otg_ipmate/linux/drivers/dwc_otg_driver.h $
 * $Revision: #1 $
 * $Date: 2005/07/07 $
 * $Change: 510275 $
 *
 * Synopsys HS OTG Linux Software Driver and documentation (hereinafter,
 * "Software") is an Unsupported proprietary work of Synopsys, Inc. unless
 * otherwise expressly agreed to in writing between Synopsys and you.
 * 
 * The Software IS NOT an item of Licensed Software or Licensed Product under
 * any End User Software License Agreement or Agreement for Licensed Product
 * with Synopsys or any supplement thereto. You are permitted to use and
 * redistribute this Software in source and binary forms, with or without
 * modification, provided that redistributions of source code must retain this
 * notice. You may not view, use, disclose, copy or distribute this file or
 * any information contained herein except pursuant to this license grant from
 * Synopsys. If you do not agree with this notice, including the disclaimer
 * below, then you are not authorized to use the Software.
 * 
 * THIS SOFTWARE IS BEING DISTRIBUTED BY SYNOPSYS SOLELY ON AN "AS IS" BASIS
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE HEREBY DISCLAIMED. IN NO EVENT SHALL SYNOPSYS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * ========================================================================== */

#if !defined(__DWC_OTG_DRIVER_H__)
#define __DWC_OTG_DRIVER_H__

/** @file
 * This file contains the interface to the Linux driver.
 */
#include "dwc_otg_cil.h"

/* Type declarations */
struct dwc_otg_pcd;
struct dwc_otg_hcd;

/**
 * This structure is a wrapper that encapsulates the driver components used to
 * manage a single DWC_otg controller.
 */
typedef struct dwc_otg_device
{
    /** Base address returned from ioremap() */
    void *base;
    
    /** irq number and register size. */
    // winder added.
    int irq;
    int reg_size;

    /** Pointer to the core interface structure. */
    dwc_otg_core_if_t *core_if;

    /** Register offset for Diagnostic API.*/
    uint32_t reg_offset;

    /** Pointer to the PCD structure. */
    struct dwc_otg_pcd *pcd;

    /** Pointer to the HCD structure. */
    struct dwc_otg_hcd *hcd;

    /** Flag to indicate whether the common IRQ handler is installed. */
    uint8_t common_irq_installed;

} dwc_otg_device_t;

/*--------------------------------------------------------------------------*/
#ifdef LINUX24 // 20070316, winder added.

#if DEBUG
#define dev_dbg(fake, format, arg...) printk(KERN_DEBUG __FILE__ ":%d: " format "\n" , __LINE__, ## arg)
#else // DEBUG
#define dev_dbg(fake, format, arg...)
#endif // DEBUG

#define dev_err(fake, format, arg...) printk(KERN_ERR __FILE__ ": " format "\n" , ## arg)

extern void *dma_alloc_noncoherent26(size_t size, dma_addr_t * dma_handle, int gfp);
extern void *dma_alloc_coherent26(size_t size, dma_addr_t * dma_handle, int gfp);
extern void dma_free_noncoherent26(size_t size, void *vaddr, dma_addr_t dma_handle);
extern void dma_free_coherent26(size_t size, void *vaddr, dma_addr_t dma_handle);

#endif // LINUX24
/*--------------------------------------------------------------------------*/

#endif
