/******************************************************************************
**
** FILE NAME    : dwc_otg_ifx.h
** PROJECT      : Twinpass/Danube
** MODULES      : DWC OTG USB
**
** DATE         : 12 April 2007
** AUTHOR       : Sung Winder
** DESCRIPTION  : Platform specific initialization.
** COPYRIGHT    :       Copyright (c) 2007
**                      Infineon Technologies AG
**                      2F, No.2, Li-Hsin Rd., Hsinchu Science Park,
**                      Hsin-chu City, 300 Taiwan.
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
** HISTORY
** $Date          $Author         $Comment
** 12 April 2007   Sung Winder     Initiate Version
*******************************************************************************/
#if !defined(__DWC_OTG_IFX_H__)
#define __DWC_OTG_IFX_H__

#ifdef LINUX24
#include <asm/addrspace.h>
#include <asm/bitops.h>
#include <asm/io.h>
#endif // LINUX24

// 20070316, winder added.
#ifndef SZ_256K
#define SZ_256K                         0x00040000
#endif

extern void dwc_otg_power_on (void);

// 20070316, winder added.
#if defined(CONFIG_TWINPASS) || defined(CONFIG_DANUBE)
/* FIXME: The current Linux-2.6 do not have these header files, but anyway, we need these. */
// #include <asm/danube/danube.h>
// #include <asm/ifx/irq.h>

/* winder, I used the Danube parameter as default. *
 * We could change this through module param.      */
#define IFX_USB_IOMEM_BESE 0x1e101000
#define IFX_USB_IOMEM_SIZE SZ_256K
#define IFX_USB_IRQ 54

/**
 * This function is called to set correct clock gating and power.
 * For Twinpass/Danube board.
 */
#ifndef DANUBE_RCU_BASE_ADDR
#define DANUBE_RCU_BASE_ADDR            (0xBF203000)
#endif

#ifndef DANUBE_CGU
#define DANUBE_CGU                          (0xBF103000)
#endif
#ifndef DANUBE_CGU_IFCCR
/***CGU Interface Clock Control Register***/
#define DANUBE_CGU_IFCCR                        ((volatile u32*)(DANUBE_CGU+ 0x0018))
#endif

#ifndef DANUBE_PMU
#define DANUBE_PMU                              (KSEG1+0x1F102000)
#endif
#ifndef DANUBE_PMU_PWDCR
/* PMU Power down Control Register */
#define DANUBE_PMU_PWDCR                        ((volatile u32*)(DANUBE_PMU+0x001C))
#endif


#define DANUBE_RCU_UBSCFG  ((volatile u32*)(DANUBE_RCU_BASE_ADDR + 0x18))
#define DANUBE_USBCFG_HDSEL_BIT    11	// 0:host, 1:device
#define DANUBE_USBCFG_HOST_END_BIT 10	// 0:little_end, 1:big_end
#define DANUBE_USBCFG_SLV_END_BIT  9	// 0:little_end, 1:big_end

/* FIXME: please make the name unified!! winder. */
#ifdef CONFIG_TWINPASS
extern void mask_and_ack_twinpass_irq (unsigned int irq_nr);
#define mask_and_ack_ifx_irq mask_and_ack_twinpass_irq
#else // for DANUBE.
extern void mask_and_ack_danube_irq (unsigned int irq_nr);
#define mask_and_ack_ifx_irq mask_and_ack_danube_irq
#endif // LINUX24

#elif defined(CONFIG_AMAZON_SE) // defined(CONFIG_TWINPASS) || defined(CONFIG_DANUBE)
#include <asm/amazon_se/amazon_se.h>
#include <asm/amazon_se/irq.h>

/* winder, I used the Danube parameter as default. *
 * We could change this through module param.      */
#define IFX_USB_IOMEM_BESE 0x1e101000
#define IFX_USB_IOMEM_SIZE SZ_256K
#define IFX_USB_IRQ 31

/**
 * This function is called to set correct clock gating and power.
 * For Twinpass/Danube board.
 */
#define AMAZON_SE_RCU_UBSCFG  ((volatile u32*)(AMAZON_SE_RCU_BASE_ADDR + 0x18))
#define AMAZON_SE_USBCFG_HDSEL_BIT    11	// 0:host, 1:device
#define AMAZON_SE_USBCFG_HOST_END_BIT 10	// 0:little_end, 1:big_end
#define AMAZON_SE_USBCFG_SLV_END_BIT  9 	// 0:little_end, 1:big_end

#define mask_and_ack_ifx_irq mask_and_ack_amazon_se_irq
extern void mask_and_ack_amazon_se_irq (unsigned int irq_nr);

#else // CONFIG_AMAZON_SE
#error "Please choose one platform!!"
#endif // CONFIG_AMAZON_SE

#ifdef LINUX24
#define dev_set_drvdata(_dev, _data) dwc_set_drvdata(_data)
#define dev_get_drvdata(_dev) dwc_get_drvdata()
#endif // LINUX24

#endif //__DWC_OTG_IFX_H__
