/* Masquerade.  Simple mapping which alters range to a local IP address
   (depending on route). */
#include <linux/config.h>
#include <linux/types.h>
#include <linux/ip.h>
#include <linux/timer.h>
#include <linux/module.h>
#include <linux/netfilter.h>
#include <net/protocol.h>
#include <net/checksum.h>
#include <linux/netfilter_ipv4.h>
#include <linux/netfilter_ipv4/ip_nat_rule.h>
#include <linux/netfilter_ipv4/ip_tables.h>

/* 0611122: Sumedh. For STUN support.
 * For ALL the UDP packets that hit this target, try to search if we find a STUN session 
 * with the same local ip:local port and stun server port. If found, then ensure that the 
 * binding that was obtained during stun session is maintained for this connection as well
 * hint: required for RTP to work for a VoIP call!
 */
#if 0 //Not used
/* 0611151: Sumedh. 
 * Maintains a table that is used as follows.
 * Add localip:localport to table during init_conntrack if dest == stun server port
 * when any pkt is being MASQ, check table. if found, but no stun ctrack and this pkt dest not stun DROP!
 */
#endif

#ifdef NAT_STUN_SUPPORT
#include <linux/ip.h>
#include <linux/udp.h>
#endif


#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

#if 0
/* 0611151 */
#ifdef NAT_STUN_SUPPORT
#define MAX_MAPPINGS 500
struct masq_list {
	unsigned int localip;
	unsigned short localport;
	unsigned short wanport; //not used currenly!!!
}mapping_list[MAX_MAPPINGS];
int curr_count=0;
#endif
/* 0611151 end */
#endif

extern struct ip_conntrack_tuple_hash * ip_conntrack_find_get_half(const struct ip_conntrack_tuple *, unsigned short);
int stun_server_port=3478;
int stun_server_port2=3479;

/* Lock protects masq region inside conntrack */
static DECLARE_RWLOCK(masq_lock);
DECLARE_RWLOCK_EXTERN(ip_conntrack_lock);

/* FIXME: Multiple targets. --RR */
static int
masquerade_check(const char *tablename,
		 const struct ipt_entry *e,
		 void *targinfo,
		 unsigned int targinfosize,
		 unsigned int hook_mask)
{
	const struct ip_nat_multi_range *mr = targinfo;

	if (strcmp(tablename, "nat") != 0) {
		DEBUGP("masquerade_check: bad table `%s'.\n", tablename);
		return 0;
	}
	if (targinfosize != IPT_ALIGN(sizeof(*mr))) {
		DEBUGP("masquerade_check: size %u != %u.\n",
		       targinfosize, sizeof(*mr));
		return 0;
	}
	if (hook_mask & ~(1 << NF_IP_POST_ROUTING)) {
		DEBUGP("masquerade_check: bad hooks %x.\n", hook_mask);
		return 0;
	}
	if (mr->range[0].flags & IP_NAT_RANGE_MAP_IPS) {
		DEBUGP("masquerade_check: bad MAP_IPS.\n");
		return 0;
	}
	if (mr->rangesize != 1) {
		DEBUGP("masquerade_check: bad rangesize %u.\n", mr->rangesize);
		return 0;
	}
	return 1;
}

static unsigned int
masquerade_target(struct sk_buff **pskb,
		  unsigned int hooknum,
		  const struct net_device *in,
		  const struct net_device *out,
		  const void *targinfo,
		  void *userinfo)
{
	struct ip_conntrack *ct;
	enum ip_conntrack_info ctinfo;
	const struct ip_nat_multi_range *mr;
	struct ip_nat_multi_range newrange;
	u_int32_t newsrc;
	struct rtable *rt;
	struct rt_key key;
	union ip_conntrack_manip_proto mymin, mymax;

#ifdef NAT_STUN_SUPPORT
	struct udphdr *udph;
	struct ip_conntrack_tuple_hash *h=NULL;
	int port=0, dport=0, sport=0;
	struct iphdr *iph = (*pskb)->nh.iph;
	struct ip_conntrack_tuple to_find;
	int wan_port=0, i=0, found = -1;
	
	//memset to zero
	memset(&to_find, 0, sizeof(to_find));
	mr = targinfo;
	if(iph->protocol == 0x11) {
                udph = (void*)iph+iph->ihl*4;
                port = (udph->source);
		dport = udph->dest;
		sport = udph->source;

		to_find.src.u.all = to_find.dst.u.all = 0;

		to_find = ((struct ip_conntrack_tuple){ { iph->saddr, { .udp = { port } } },
			     			{ 0, { 0 },
					      	  IPPROTO_UDP } });

		READ_LOCK(&masq_lock);
		h = ip_conntrack_find_get_half(&to_find, stun_server_port);	
		if (h == NULL)
			h = ip_conntrack_find_get_half(&to_find, stun_server_port2);	

		if (h !=NULL && (ntohs(udph->dest)==stun_server_port || ntohs(udph->dest) == stun_server_port2)) {
			/*Sumedh: This means there is already a STUN CT, if the IPs dont match then delete old one! */
			READ_LOCK(&ip_conntrack_lock);
			if ( iph->daddr != (h->ctrack->tuplehash[IP_CT_DIR_REPLY].tuple.src.ip) ) {
				//Delete the old conntrack!
				clean_from_lists(h->ctrack);
			}
			READ_UNLOCK(&ip_conntrack_lock);

		}
		if(h == NULL) {
			if(dport == htons(stun_server_port) || dport == htons(stun_server_port2)) {
				h = ip_conntrack_find_get_half(&to_find, 0);	
				//if(h == NULL && sport == htons(5060) ) {
				//	//wan_port = 24000; 
				//	wan_port = 25000; //24000-24999 used by MASQ for SIP ALG. above 24999 STUN+SIP
				//	while(is_wan_port_used(wan_port,5060))
				//		wan_port++;	
				//}
			}
		}
		READ_UNLOCK(&masq_lock);
		if (h!=NULL) {
			READ_LOCK(&ip_conntrack_lock);
			wan_port= ntohs(h->ctrack->tuplehash[IP_CT_DIR_REPLY].tuple.dst.u.udp.port);
			READ_UNLOCK(&ip_conntrack_lock);
		}
	}
#endif
	IP_NF_ASSERT(hooknum == NF_IP_POST_ROUTING);

	/* FIXME: For the moment, don't do local packets, breaks
	   testsuite for 2.3.49 --RR */
	if ((*pskb)->sk)
		return NF_ACCEPT;

	ct = ip_conntrack_get(*pskb, &ctinfo);
	IP_NF_ASSERT(ct && (ctinfo == IP_CT_NEW || ctinfo == IP_CT_RELATED
	                    || ctinfo == IP_CT_RELATED + IP_CT_IS_REPLY));

	mr = targinfo;

	key.dst = (*pskb)->nh.iph->daddr;
	key.src = 0; /* Unknown: that's what we're trying to establish */
	key.tos = RT_TOS((*pskb)->nh.iph->tos)|RTO_CONN;
	key.oif = 0;
#ifdef CONFIG_IP_ROUTE_FWMARK
	key.fwmark = (*pskb)->nfmark;
#endif
	if (ip_route_output_key(&rt, &key) != 0) {
#ifndef CONFIG_BSP_LESS_MEMORY_USAGE
	/* Avoid doing a Rate Limiting */
                /* Funky routing can do this. */
                if (net_ratelimit())
                        printk("MASQUERADE:"
                               " No route: Rusty's brain broke!\n");
#endif
                return NF_DROP;
        }
        if (rt->u.dst.dev != out) {
                if (net_ratelimit())
                        printk("MASQUERADE:"
                               " Route sent us somewhere else.\n");
			ip_rt_put(rt);
		return NF_DROP;
	}

	newsrc = rt->rt_src;
	DEBUGP("newsrc = %u.%u.%u.%u\n", NIPQUAD(newsrc));
	ip_rt_put(rt);

	WRITE_LOCK(&masq_lock);
	ct->nat.masq_index = out->ifindex;
	WRITE_UNLOCK(&masq_lock);

#ifdef NAT_STUN_SUPPORT
	/* Transfer from original range. */
	if(wan_port) {
		memset(&mymin, 0, sizeof(mymin));
		memset(&mymax, 0, sizeof(mymax));
		mymin.udp.port = wan_port;
		mymax.udp.port = wan_port;
		newrange = ((struct ip_nat_multi_range)
				{ 1, { { mr->range[0].flags | IP_NAT_RANGE_MAP_IPS | IP_NAT_RANGE_PROTO_SPECIFIED,
				newsrc, newsrc,
				mymin, mymax } } });
	}
	else
#endif
	{
	newrange = ((struct ip_nat_multi_range)
		{ 1, { { mr->range[0].flags | IP_NAT_RANGE_MAP_IPS,
			 newsrc, newsrc,
			 mr->range[0].min, mr->range[0].max } } });
	}

	/* Hand modified range to generic setup. */
	return ip_nat_setup_info(ct, &newrange, hooknum);
}

static inline int
device_cmp(struct ip_conntrack *i, void *ifindex)
{
	int ret;

	READ_LOCK(&masq_lock);
	ret = (i->nat.masq_index == (int)(long)ifindex);
	READ_UNLOCK(&masq_lock);

	return ret;
}

static int masq_device_event(struct notifier_block *this,
			     unsigned long event,
			     void *ptr)
{
	struct net_device *dev = ptr;

	if (event == NETDEV_DOWN) {
		/* Device was downed.  Search entire table for
		   conntracks which were associated with that device,
		   and forget them. */
		IP_NF_ASSERT(dev->ifindex != 0);
 
		ip_ct_iterate_cleanup(device_cmp, (void *)(long)dev->ifindex);
	}

	return NOTIFY_DONE;
}


static int masq_inet_event(struct notifier_block *this,
			   unsigned long event,
			   void *ptr)
{
	struct net_device *dev = ((struct in_ifaddr *)ptr)->ifa_dev->dev;

	if (event == NETDEV_DOWN) {
		/* IP address was deleted.  Search entire table for
		   conntracks which were associated with that device,
		   and forget them. */
		IP_NF_ASSERT(dev->ifindex != 0);

		ip_ct_iterate_cleanup(device_cmp, (void *)(long)dev->ifindex);
	}

	return NOTIFY_DONE;
}

static struct notifier_block masq_dev_notifier = {
	.notifier_call  = masq_device_event,
};

static struct notifier_block masq_inet_notifier = {
	.notifier_call = masq_inet_event
};

static struct ipt_target masquerade
= { { NULL, NULL }, "MASQUERADE", masquerade_target, masquerade_check, NULL,
    THIS_MODULE };

static int __init init(void)
{
	int ret;

	ret = ipt_register_target(&masquerade);

	if (ret == 0) {
		/* Register for device down reports */
		register_netdevice_notifier(&masq_dev_notifier);
		/* Register IP address change reports */
		register_inetaddr_notifier(&masq_inet_notifier);
	}

	return ret;
}

static void __exit fini(void)
{
	ipt_unregister_target(&masquerade);
	unregister_netdevice_notifier(&masq_dev_notifier);
	unregister_inetaddr_notifier(&masq_inet_notifier);	
}

module_init(init);
module_exit(fini);
MODULE_LICENSE("GPL");
